<?php

if ( !defined('ABSPATH') ) exit;

if ( !class_exists('FILO_Admin_List_Table_Finadoc') ) :
	
/**
 * FILO_Admin_List_Table_Finadoc -> extend abstract-class-wc-admin-list-table.php
 * 
 * List tables: finadoc
 *
 * @package     Filogy/Admin
 * @subpackage 	Financials
 * @category    Admin
 */

if ( ! class_exists( 'WC_Admin_List_Table_Orders', false ) ) {
	include_once(WC()->plugin_path() .  '/includes/admin/list-tables/class-wc-admin-list-table-orders.php');
}

class FILO_Admin_List_Table_Finadoc extends WC_Admin_List_Table_Orders {

	/**
	 * Post type.
	 *
	 * @var string
	 */
	protected $list_table_type = ''; //set it in the constructor according to the actual finadoc type

	/**
	 * Constructor.
	 */
	public function __construct() {

		// this is needed for displaying our custom finadocs like shop orders in the order list screen 
		// we have to set our custom filogy document post type into $this->list_table_type variable, for __construct() of abstract-class-wc-admin-list-table.php be able to set some (4) post type specific filters for the actual filogy post type
		// all other functionality is the same than the extended WC_Admin_List_Table_Orders class.
		
		global $filo_post_types_financial_documents;
		
		wsl_log(null, 'class-filo-admin-list-table-finadoc.php __construct 0:' .  wsl_vartotext( '' ));
		
		if ( isset($_GET['post_type']) and in_array( $_GET['post_type'], $filo_post_types_financial_documents) ) {
			
			add_filter( 'woocommerce_admin_order_actions', array( $this, 'render_financial_document_actions' ), 10, 2 ); //ADD RaPe //Add custom actions
			
		}
		
	}
	
	/**
	 * @param  string $column
	 */
	public function render_financial_document_actions( $actions, $the_order ) {
		global $post, $typenow; //, $woocommerce, $the_order;
		
		do_action( 'filo_admin_order_actions_start', $the_order );
		
		wsl_log(null, 'class-filo-admin-list-table-finadoc.php render_financial_document_actions 0: ' . wsl_vartotext( '' )); //long
		//wsl_log(null, 'class-filo-admin-list-table-finadoc.php render_financial_document_actions $the_order: ' . wsl_vartotext($the_order)); //long
		
		$order_type_object = get_post_type_object( $the_order->get_doc_type() ); //$order_type_object in WC v2.4.10
		$doc_type_name = $order_type_object->labels->singular_name;
		
		$new_actions = array();
		if ( FILO_TYPE == 'filo_invoice_type' ) {
			// display print buttons of delivery note and invoice pseudo doc types in the shop_order list
			// it is available only in FILOFY INVOICE (free and pro)
			
			$new_actions['print_pseudo_sa_deliv_note'] = array(
				'url' 		=> wp_nonce_url( home_url() . '?filo_individual_page=filo_generate_pdf&doc_id=' . $post->ID , 'filo_generate_pdf_' . $post->ID, 'filo_nonce' )  . '&filo_usage=doc_view&filo_pseudo_doc_type=filo_sa_deliv_note',
				'name' 		=> __( 'Print Delivery Note', 'filo_text' ),
				'action' 	=> "print",
				'target'	=> "_blank",
			);
			
			$new_actions['print_pseudo_sa_invoice'] = array(
				'url' 		=> wp_nonce_url( home_url() . '?filo_individual_page=filo_generate_pdf&doc_id=' . $post->ID , 'filo_generate_pdf_' . $post->ID, 'filo_nonce' )  . '&filo_usage=doc_view&filo_pseudo_doc_type=filo_sa_invoice',
				'name' 		=> __( 'Print Invoice', 'filo_text' ),
				'action' 	=> "print",
				'target'	=> "_blank",
			);
		}
		
		$new_actions = apply_filters( 'filo_admin_order_actions', $new_actions, $the_order );
		
		wsl_log(null, 'class-filo-admin-list-table-finadoc.php render_financial_document_actions $new_actions: ' . wsl_vartotext($new_actions));
		
		/*foreach ( $new_actions as $action ) {
			printf( '<a class="button tips %s" href="%s" data-tip="%s" target="%s" >%s</a>', esc_attr( isset($action['action']) ? $action['action'] : '' ), esc_url( isset($action['url']) ? $action['url'] : '' ), esc_attr( isset($action['name']) ? $action['name'] : '' ), esc_attr( isset($action['target']) ? $action['target'] : ''), esc_attr( isset($action['name']) ? $action['name'] : ''  ) ); //MODIFY RaPe
		}*/
		
		$actions = array_merge( $actions, $new_actions );
		
		//remove WC original order actions if it is not a shop_order
		
		wsl_log(null, 'class-filo-admin-list-table-finadoc.php render_financial_document_actions $actions: ' . wsl_vartotext($actions));
		
		do_action( 'filo_admin_order_actions_end', $the_order );
		
		return $actions;
		
	}
	
}
endif;