<?php

if ( !defined('ABSPATH') ) exit;

if ( !class_exists('FILO_Settings_Financials') ) :
if ( !class_exists('FILO_Settings_Page') ) require_once( 'class-filo-settings-page.php' );

/**
 * Generate Financial Settings Tab on WooCommerce Settings Page
 * 
 * @package     Filogy/Admin/Settings
 * @subpackage 	Financials
 * @category    Admin/Settings
 * 
 */
class FILO_Settings_Financials extends FILO_Settings_Page {


	/**
	 * ...fields
	 *
	 * @var array
	 */
	private static $table_fields = array();

	private static $table_values = array();

	/**
	 * construct
	 */	
	 	 
	 public function __construct() {
	 	
	 	wsl_log(null, 'class-filo-settings-financials.php __construct 0: ' . wsl_vartotext( '' ));
		
		$this->id    = 'financials';
		$this->label = __( 'Financials', 'filo_text' );

		add_filter( 'woocommerce_settings_tabs_array', array( $this, 'add_settings_page' ), 20 );
		add_action( 'woocommerce_sections_' . $this->id, array( $this, 'output_sections' ) );
		add_action( 'woocommerce_settings_' . $this->id, array( $this, 'output' ) );
		add_action( 'woocommerce_settings_save_' . $this->id, array( $this, 'save' ) );

	}

	/**
	 * get_sections
	 */
	public function get_sections() {

		//We have only one sub-tab inside of Financial settings tab
		$sections = array(
			'' => __( 'General', 'filo_text' ), //code of first tab is empty ('')
		);

		return apply_filters( 'filo_get_sections_' . $this->id, $sections ); //filo_get_sections_financials
	}


	/**
	 * get_field_settings
	 * (this function declare the content of the specific setting section page (sub-tab))
	 */
	public function get_field_settings( $current_section = '' ) { //get_settings() was earlyer, but it was called by something (so get_settings maybe a kind a "reversed function name")
	
		wsl_log(null, 'class-filo-settings-financials.php get_field_settings $current_section: ' . wsl_vartotext($current_section));
	
		if ($current_section == '') {
			$settings = apply_filters( 'filo_financial_settings', array(
	
				array( 'type' => 'sectionend', 'id' => 'general_settings' ),
				array( 'title' => __( 'General Settings', 'filo_text' ), 'type' => 'title', 'desc' => __( 'Set general financial settings.', 'filo_text' ), 'id' => 'general_settings' ),

				array(
					'title'    => __( 'Order document number display format', 'filo_text' ),
					'desc'     => __( 'Filogy has an advanced order numbering possibility. So every order has the original WooCommerce order number and another number generated by Filogy. You can choose which number would you like to display. The chosen order number will be displayed on the front-end too.', 'filo_text' ),
					'id'       => 'filo_order_number_display_format',
					'css'      => 'min-width:150px;',
					'default'  => 'filo_only',
					'type'     => 'select',
					'options'  => array(
						'filo_only'     => __( 'Filogy order format only', 'filo_text' ),
						'wc_only'      => __( 'WooCommerce original order format only', 'filo_text' ),
						'filo_and_wc'   => __( 'Filogy order format plus WooCommerce original order format in brackets', 'filo_text' ),
					),
					'desc_tip' =>  true,
				),
				
				array(
					'title'         => __( 'Strict settings validation', 'filo_text' ),
					'desc'          => __( 'Clear this checkbox if you want to hide warning on top of pages, if the mandatory settings are done, but some optional settings have not been set yet.', 'filo_text' ),
					'id'            => 'filo_scrict_settings_validaton',
					'default'       => 'yes',
					'type'          => 'checkbox',
					//'checkboxgroup' => 'filo_display_options',
					'autoload'      => false,
					'desc_tip' =>  true,
				),
				
				array( 'type' => 'sectionend', 'id' => 'email_template_options' ),
	
			));
		}
		
		

		return apply_filters( 'filo_get_settings_' . $this->id, $settings, $current_section );
	}

	/**
	 * output
	 */
	public function output() {
		global $current_section;

		wsl_log(null, 'class-filo-settings-financials.php output $current_section: ' . wsl_vartotext($current_section));

		$settings = $this->get_field_settings( $current_section );

 		WC_Admin_Settings::output_fields( $settings );
		
	}
	
	/**
	 * save
	 */
	public function save() {
		
		global $current_section;

		wsl_log(null, 'class-filo-settings-financials.php save $_POST: ' . wsl_vartotext($_POST));
		wsl_log(null, 'class-filo-settings-financials.php save $current_section: ' . wsl_vartotext($current_section));

		if ( ! $current_section ) { //if section is empty (first sub tab), save the general "Document Options" tab

			$settings = $this->get_field_settings();
			FILO_Admin_Settings::save_fields( $settings );
			
			wsl_log(null, 'class-filo-settings-financials.php save $settings: ' . wsl_vartotext($settings));
			
		}	

		//save_sequence_details
		if ( isset( $_POST['field_sequence_id'] ) ) {
			
			//case is not a classical financial document, but a "semy" financial document, because it hase sequencial numbers, like real finadocs.
			//so we have include FILO_Document_Case for handling sequences
			
			//include FILO_Document_Case and the class from which it is inherited
			//include_once( WC()->plugin_path() . '/includes/' . 'abstracts/abstract-wc-email.php' );	//WC v2.2.6
			include_once( WC()->plugin_path() . '/includes/' . 'emails/class-wc-email.php' ); 			//WC v2.4.10
			include_once( FILOFW()->plugin_path() . '/includes/abstracts/abstract-filo-document.php' );
			$file = FILO()->plugin_path() . '/includes/documents/class-filo-document-case.php';

			//The included class-filo-document-case.php returns the FILO_Document_Case object due to the last "return new FILO_Document_Case();" line.			
			$document_case = include( $file );

			$document_case->save_sequence_details();			
			
		}
		
		//clrare cached settings check state
		update_option( 'filo_settings_ok_none_strict', null);
		update_option( 'filo_settings_ok_strict', null);
		update_option( 'filo_settings_ok_calc_date', null);
					
	}

}
endif;

return new FILO_Settings_Financials();
